// Created by bryankeller on 11/28/18.
// Copyright © 2018 Airbnb, Inc.

// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import MagazineLayout
import UIKit

// MARK: - DataSource

final class DataSource: NSObject {

  var sectionInfos = [SectionInfo]()
}

// MARK: UICollectionViewDataSource

extension DataSource: UICollectionViewDataSource {

  func numberOfSections(in collectionView: UICollectionView) -> Int {
    return sectionInfos.count
  }

  func collectionView(
    _ collectionView: UICollectionView,
    numberOfItemsInSection section: Int)
    -> Int
  {
    return sectionInfos[section].itemInfos.count
  }

  func collectionView(
    _ collectionView: UICollectionView,
    cellForItemAt indexPath: IndexPath)
    -> UICollectionViewCell
  {
    let cell = collectionView.dequeueReusableCell(
      withReuseIdentifier: Cell.description(),
      for: indexPath) as! Cell
    let itemInfo = sectionInfos[indexPath.section].itemInfos[indexPath.item]
    cell.set(itemInfo)
    return cell
  }
}

// MARK: DataSourceCountsProvider

extension DataSource: DataSourceCountsProvider {

  var numberOfSections: Int {
    return sectionInfos.count
  }

  func numberOfItemsInSection(withIndex sectionIndex: Int) -> Int {
    return sectionInfos[sectionIndex].itemInfos.count
  }

}
