// Created by bryankeller on 11/28/18.
// Copyright © 2018 Airbnb, Inc.

// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import MagazineLayout
import UIKit

// MARK: - ViewController

final class ViewController: UIViewController {

  // MARK: Internal

  override func viewDidLoad() {
    super.viewDidLoad()

    navigationItem.title = "MagazineLayout Example"
    navigationItem.rightBarButtonItem = UIBarButtonItem(
      barButtonSystemItem: .add,
      target: self,
      action: #selector(addButtonTapped))

    view.addSubview(collectionView)

    collectionView.translatesAutoresizingMaskIntoConstraints = false
    NSLayoutConstraint.activate([
      collectionView.leadingAnchor.constraint(equalTo: view.leadingAnchor),
      collectionView.trailingAnchor.constraint(equalTo: view.trailingAnchor),
      collectionView.topAnchor.constraint(equalTo: view.topAnchor),
      collectionView.bottomAnchor.constraint(equalTo: view.bottomAnchor),
    ])
  }

  // MARK: Private

  private lazy var collectionView: UICollectionView = {
    let layout = MagazineLayout()
    let collectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
    collectionView.register(Cell.self, forCellWithReuseIdentifier: Cell.description())
    collectionView.isPrefetchingEnabled = false
    collectionView.dataSource = dataSource
    collectionView.delegate = self
    collectionView.backgroundColor = .white
    collectionView.contentInsetAdjustmentBehavior = .always
//    collectionView.contentInset = .zero
    #warning("This fixes the problem when inserted cell is on top of existing cells")
//        collectionView.contentInset = UIEdgeInsets(top: 0, left: 1, bottom: 0, right: 1)
    return collectionView
  }()

    private lazy var dataSource: DataSource = {
        let section0 = SectionInfo(itemInfos: [])
        let section1 = SectionInfo(itemInfos: [
            ItemInfo(
                sizeMode: MagazineLayoutItemSizeMode(
                    widthMode: .fullWidth(respectsHorizontalInsets: true),
                    heightMode: .static(height: 60)),
                text: "Existing cell",
                color: Colors.red),
            ])
        let dataSource = DataSource()
        dataSource.sectionInfos = [section0, section1]
        return dataSource
    }()

  @objc
  private func addButtonTapped() {
    let itemInfo = ItemInfo(
        sizeMode: MagazineLayoutItemSizeMode(
            widthMode: .fullWidth(respectsHorizontalInsets: true),
            heightMode: .static(height: 30)),
        text: "Inserted cell",
        color: Colors.blue)

    self.collectionView.performBatchUpdates({

        dataSource.sectionInfos[0].itemInfos.insert(itemInfo, at: 0)
        self.collectionView.insertItems(at: [IndexPath(item: 0, section: 0)])
    }, completion: nil)
  }

}

// MARK: UICollectionViewDelegateMagazineLayout

extension ViewController: UICollectionViewDelegateMagazineLayout {

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    sizeModeForItemAt indexPath: IndexPath)
    -> MagazineLayoutItemSizeMode
  {
    return dataSource.sectionInfos[indexPath.section].itemInfos[indexPath.item].sizeMode
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    visibilityModeForHeaderInSectionAtIndex index: Int)
    -> MagazineLayoutSupplementaryViewVisibilityMode
  {
    return .hidden
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    visibilityModeForFooterInSectionAtIndex index: Int)
    -> MagazineLayoutSupplementaryViewVisibilityMode
  {
    return .hidden
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    visibilityModeForBackgroundInSectionAtIndex index: Int)
    -> MagazineLayoutBackgroundVisibilityMode
  {
    return .hidden
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    horizontalSpacingForItemsInSectionAtIndex index: Int)
    -> CGFloat
  {
    return 0
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    verticalSpacingForElementsInSectionAtIndex index: Int)
    -> CGFloat
  {
    return 0
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    insetsForItemsInSectionAtIndex index: Int)
    -> UIEdgeInsets
  {
    return .zero
  }

  func collectionView(
    _ collectionView: UICollectionView,
    layout collectionViewLayout: UICollectionViewLayout,
    insetsForSectionAtIndex index: Int)
    -> UIEdgeInsets
  {
    return .zero
  }
}
